# -*- coding: utf-8 -*-
""" Biogas Channel  >  project.core.fh_admin.renderers
    @author: Frankhood Business Solutions <info@frankhood.it>
    @author: Gaetano D'Onghia <g.donghia@frankhood.it>
"""
from __future__ import absolute_import, print_function, unicode_literals

import csv
import logging

from rest_framework import generics
from rest_framework.settings import api_settings
from rest_framework_csv import renderers as csv_renderers
from six import StringIO, text_type

try:
    from six import PY2
except ImportError:
    import sys
    PY2 = sys.version_info[0] == 2


logger = logging.getLogger(__name__)


class FHCSVRenderer(csv_renderers.CSVRenderer):

    csv_delimiter = b';'
    csv_dialect = 'excel'
    csv_quotechar = b'"'

    def get_custom_headers(self):
        return self.custom_headers

    def write_extra_rows(self, csv_writer):
        # CUSTOM X Inserire righe alla fine
        # csv_writer.writerow([None,None,None,None])
        pass

    def render(self, data, media_type=None, renderer_context=None):
        """
        Renders serialized *data* into CSV.
        """
        if data is None:
            return ''

        if not isinstance(data, list):
            data = [data]

        table = self.tablize(data)
        csv_buffer = StringIO()
        csv_writer = csv.writer(csv_buffer, dialect=self.csv_dialect, delimiter=self.csv_delimiter, quotechar=self.csv_quotechar)
        for row in table:
            # Assume that strings should be encoded as UTF-8
            csv_writer.writerow([
                elem.encode('utf-8') if isinstance(elem, text_type) and PY2 else elem
                for elem in row
            ])
        logger.info("render DATA: %s" % (data))

        self.write_extra_rows(csv_writer)

        return csv_buffer.getvalue()

    def tablize(self, data):
        _headers = self.get_custom_headers()
        if data:
            ordered_headers = [x if isinstance(x, basestring) else x[0] for x in _headers]
            formatted_headers = [x if isinstance(x, basestring) else x[1] for x in _headers]
            data = self.flatten_data(data)
            data.header = data.header or ordered_headers
            if not data.header:
                headers = set()
                for item in data:
                    headers.update(list(item.keys()))
                data.header = sorted(headers)
            rows = []
            for item in data:
                row = []
                for key in data.header:
                    row.append(item.get(key, None))
                rows.append(row)
            return [formatted_headers] + rows


class AdminFHCSVRenderer(FHCSVRenderer):
    """
    Permette di modificare i campi del CSV e la sua formattazione
        @param custom_headers può essere :
            - None (list_display dell'admin ^ field del model)
            - Lista di field passategli
            - Lista di tuple in cui per ogni chiave viene specificata la sua formattazione
    """
