# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import logging

from django.conf import settings
from django.utils.encoding import smart_text
from django.utils.importlib import import_module

from django.contrib.sessions.middleware import SessionMiddleware

logger = logging.getLogger(__name__)


class MiddlewareResponseInjectP3P(object):

    P3P_COMPACT = 'CP=HONK'

    def __init__(self):
        self.process_response = self.inject

    def inject(self, request, response):
        response['P3P'] = self.P3P_COMPACT
        response['p3p'] = self.P3P_COMPACT
        return response


class SessionMiddlewareWithSSID(SessionMiddleware):

    ssid_key = "ssid"
    only_safari = True

    def process_request(self, request):
        engine = import_module(settings.SESSION_ENGINE)
        patched = False
        session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME, None)
        if not session_key:
            # Controllo se il browser è safari, nel caso lo "patcho"
            ua = smart_text(request.META.get('HTTP_USER_AGENT', ''))
            if self.only_safari:
                if ua.find('Safari') != -1:
                    # PATCH per accesso a request.body
                    request.body
                    session_key = request.REQUEST.get(self.ssid_key, '')
                    patched = True
            else:
                # PATCH per accesso a request.body
                request.body
                session_key = request.REQUEST.get(self.ssid_key, '')
        request.session = engine.SessionStore(session_key)
        request.session.iframe_patched = patched

try:
    import django_mobile
    from django_mobile.middleware import MobileDetectionMiddleware

    class TabletAndMobileDetectionMiddleware(MobileDetectionMiddleware):

        user_agents_test_search = u"(?:%s)" % u'|'.join((
            'up.browser', 'up.link', 'mmp', 'symbian', 'smartphone', 'midp',
            'wap', 'phone', 'windows ce', 'pda', 'mobile', 'mini', 'palm',
            'netfront', 'opera mobi', 'android',
        ))
        user_agents_exception_search = u"(?:%s)" % u'|'.join((
            'sghinofoldisopsofonsdcoalxkjvahjfajkbln',
        ))
except ImportError:
    logger.info("TabletAndMobileDetectionMiddleware cannot be installed if django_mobile is not installed")
