# -*- coding: utf-8 -*-
""" fhcore.apps.facebook.views.mixins by Frankhood
    @author: Frankhood Business Solutions
    @author: Gaetano D'Onghia
"""
from __future__ import absolute_import, print_function, unicode_literals
from django.utils.encoding import python_2_unicode_compatible
import cgi
import hmac
import time
import base64
import hashlib
import urlparse
import urllib

from django.conf import settings
from django.http import HttpResponse
from django.core.exceptions import ImproperlyConfigured
from django.template.loader import get_template
from django.template import Context
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.cache import never_cache
from django.views.generic import View, TemplateView, ListView, DetailView
from django.views.generic.base import TemplateResponse, TemplateResponseMixin,\
    ContextMixin
from django.contrib.sites.models import Site
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext as _u

from fhcore.apps.facebook.views.base import FBBaseMixin, FB_OAUTH_TEMPLATE
from django.http.response import HttpResponseRedirect

try:
    from fbconsole import Client as FBClient
    from fbconsole import OAuthException
except ImportError:
    raise ImproperlyConfigured("FH Facebook Views require the 'fbconsole' cots module")

try:
    # Django <= 1.6 backwards compatibility
    from django.utils import simplejson as json
except ImportError:
    # Django >= 1.7
    import json

import logging
logger = logging.getLogger('project')

#from fhcore.utils.lazy import LazyReverseUrl
from fhcore.apps.facebook.views.exceptions import ActionNotAllowed, NoUserException
from fhcore.views.base import JSONResponseMixin, HeadResponseMixin


class FBContextMixin(ContextMixin):
    
    def get_context_data(self, **kwargs):
        context = super(FBContextMixin, self).get_context_data(**kwargs)
        context.update({
             'fb_data' : self.facebook_data,
             'app_id' : self.fb_app_id,
             'canvas_url' : self.fb_canvas_url,
             'tab_url' : self.fb_tab_url,
             'app_name' : self.fb_app_name,
             'app_data' : self.encode_app_data()
        })
        if hasattr(self,'app_scope') and self.app_scope:
            context['app_scope']= ", ".join(self.app_scope)
        if hasattr(self,'fb_user') and self.fb_user:
            context['fb_user'] = self.fb_user
        if hasattr(self,'get_player') and self.get_player():
            context['player'] = self.get_player()
        return context


class FBUserMixin(FBBaseMixin):
    is_oauth_required = True
    oauth_template = FB_OAUTH_TEMPLATE
    app_name = settings.PROJECT_NAME
    
    def prehandle_response(self, request, *args, **kwargs):
        response = super(FBUserMixin,self).prehandle_response(request, *args, **kwargs)
        if not response and self.is_oauth_required and (not self.fb_user or self.is_token_expired()):
            # Controllo se hanno rifiutato i permessi
            #print request.REQUEST
            logger.warning("Redirect perchè non c'è FBUser ed è richiesto")
            login_template = self.get_login_template(request, *args, **kwargs)
            return HttpResponse(login_template)
        return response
    
    def prefetch_data(self, request, *args, **kwargs):
        try:
            self.get_userdata_from_facebook(request)
        except OAuthException:
            if self.is_oauth_required:
                logger.exception("Redirect perchè OAuthException")
                login_template = self.get_login_template(request, *args, **kwargs)
                return HttpResponse(login_template)
            #raise
    
    def get_userdata_from_facebook(self, request):
        self.fb_user = self.get_fb_user()
    
    def get_fb_user(self):
        if hasattr(self,'fb_user') and self.fb_user:
            return self.fb_user
        if self.fb_client:
            return self.fb_client.get('/me')
        else:
            return None
    
    def get_user_has_liked(self):
        """ @return: bool """
        has_liked = self.facebook_data.get('signed_request', {}).get('page', {}).get('liked', False)
        return has_liked

    def perform_age_check(self):
        """ @return: bool """
        user_data = self.facebook_data.get('signed_request', {}).get('user', {}).get('age', {}).get('max', 21)
        age_check = int(user_data) < 18
        return age_check

    def is_token_expired(self):
        try:
            # Faccio la chiamata + semplice del mondo per provare se il token e' scaduto
            self.fb_client.get('/me')
            return False
        except:
            return True

    def get_login_context_data(self, request=None):
        request = request or self.request
        context_data =  {'url': self.get_login_url(request),
                         'app_name' : self.app_name,
                         'fb_meta_title': self.app_name,
                         'fb_meta_description': self.app_name,}
        return context_data
        
    def get_login_template(self, request, *args, **kwargs):
        """ render a pass-through template with a specific context, 
            wich is used to redirect outside a frame.
            @return str parsed template
        """
        c = Context(self.get_login_context_data(request))
        t = get_template(self.oauth_template)
        return t.render(c)

    
    def get_protocol(self):
        return 'http://' if settings.DEBUG else 'https://'
        
    def get_login_url(self, request, *args, **kwargs):
        """ build the facebook login endpoint url with the right parameters 
            - client_id: the facebook app_id;
            - redirect_uri: the url by witch the user will be redirected;
            - scope: a list of permissions the user has to accept;
            @return str
        """
        base_url = self.oauth_dialog_url
        params = {
            'client_id': self.fb_app_id,
            'scope': ','.join(self.app_scope),
            'redirect_uri':  self.get_redirect_uri(request)}
        encoded_params = urllib.urlencode(params)
        return '%s?%s' % (base_url, encoded_params)
    
    def get_redirect_uri(self, request):
        if hasattr(self,'redirect_uri') and self.redirect_uri:
            return self.redirect_uri
        return urlparse.urljoin(
            self.get_protocol()+request.get_host(), 
            self.get_redirect_page(request))
        
    def get_redirect_page(self, request=None, **kwargs):
        return self.redirect_page   


####PLAYER REQUIRED

class FBPlayerMixin(FBUserMixin):
    player_model = None
    facebook_id_attribute = None #"fb_id"
    is_player_required = True
    can_create_player = True
    
    def get_userdata_from_facebook(self, request):
        """
        Returns the instance of self.player_model or the fb_user(FBClient instance) if not self.player_required  """
        super(FBPlayerMixin,self).get_userdata_from_facebook(request)
        self.player = self.get_player()
    
    def get_player(self):
        if hasattr(self,'player') and self.player:
            return self.player
        if hasattr(self,'fb_user') and self.fb_user:
            player_data = self.fb_user
        else:
            user_fb_id = self.facebook_data.get('signed_request', {}).get('user_id', None)
            if user_fb_id:
                player_data = {'id':user_fb_id}
            else:
                player_data = None
        if player_data:
            if self.can_create_player:
                player, created = self.player_model.objects.get_or_create_from_facebook(player_data)
                """
                if created:
                    logger.info("Created facebook player")
                else:
                    logger.info("Existing Facebook player")
                """
            else:
                player = self.player_model.objects.get_from_facebook(player_data)
        else:
            player=None
        return player
    
    def get_player_registration_url(self, request):
        raise ImproperlyConfigured("Se un utente deve potersi registrare bisogna modificare 'get_player_registration_url' per fare redirect alla registrazione")
    
    def prehandle_response(self, request, *args, **kwargs):
        response = super(FBPlayerMixin,self).prehandle_response(request, *args, **kwargs)
        if not response and (not self.get_player()):
            # Controllo se hanno rifiutato i permessi
            if self.is_player_required and not self.can_create_player:
                return HttpResponseRedirect(self.get_player_registration_url(request))
            else:
                pass
        return response   
    
    
## UTILITIES
class FBFromSessionMixin(object):
    can_set_access_token_from_request = True
    
    """
    def set_fb_user(self):
        if self.fb_client:
            self.fb_user = self.fb_client.get('/me')
    """

    def set_access_token_from_request(self):
        access_token = self.request.REQUEST.get('access_token', None)
        self.facebook_data['access_token'] = access_token
        if access_token:
            logger.info("L'access token è preso dalla GET: {0}".format(access_token))
            self.request.session['facebook_data'] = self.facebook_data
            self.fb_client = FBClient(access_token)
            #if not self.is_token_expired():
                #self.set_fb_user()
        else:
            logger.info("L'access token non c'è in GET".format(access_token))
            self.fb_client = None
    
    def load_facebook_data(self, request, *args, **kwargs):
        """ fetch the facebook 'signed request', then parses it and injects 
            a new attribute 'facebook_data' into the object.
            It also caches the data in a session parameter.
            @set ivar facebook_data
            @set request.session.facebook_data
            @return void
        """
        if request.REQUEST.has_key('signed_request'):
            signed_request = request.REQUEST['signed_request']
            facebook_data = self.parse_signed_request(signed_request)
            request.session['facebook_data'] = facebook_data
        else:
            facebook_data = request.session.get('facebook_data', {})
        self.facebook_data = facebook_data
        self.app_data = self.parse_app_data(request, facebook_data)
        access_token = facebook_data.get('access_token', None)
        if access_token:
            # logger.exception("L'access token c'è Verifichiamo che funzioni")
            self.fb_client = FBClient(access_token)
            if self.is_token_expired():
                # logger.exception("L'access toke c'è ma è sbagliato! Verifichiamo che ci sia in GET")
                if self.can_set_access_token_from_request:
                    self.set_access_token_from_request()
        else:
            # logger.exception("L'access token  non c'è! Verifichiamo che ci sia in GET")
            if self.can_set_access_token_from_request:
                self.set_access_token_from_request()

class FBNegotiationMixin(FBUserMixin):
    oauth_access_token_url = "https://graph.facebook.com/oauth/access_token"

    def trade_access_code(self, request, *args, **kwargs):
        ''' negotiate an access code 'provided by facebook with an access token.
            @note: should be used to implement specific use cases wich require
                   an authentication outside a facebook chrome.
            @return str access token
        '''
        try:
            code = request.REQUEST.get('code')
            redirect_uri = urlparse.urljoin(
                self.get_protocol() + request.get_host(), 
                self.get_redirect_page(request))
            auth_parameters = {
                'client_id': self.fb_app_id,
                'redirect_uri': redirect_uri,
                'client_secret': self.fb_app_secret,
                'code': code}
            query_string = urllib.urlencode(auth_parameters) 
            endpoint_url = self.oauth_access_token_url + '?%s' % query_string
            access_token_raw = urllib.urlopen(endpoint_url).read()
            access_token = cgi.parse_qs(access_token_raw).get('access_token')[0]
        except Exception, ex:
            logger.error("Cant complete the access_key request: %s" % ex)
            if settings.DEBUG:
                raise
        return locals().get('access_token', None)  