import logging
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.db.models import get_model
    
logger=logging.getLogger("fhcore")

PLAYER_MODEL = getattr(settings,'PLAYER_MODEL','game.Player')
IS_PLAYER_REQUIRED = getattr(settings,'IS_PLAYER_REQUIRED',True)

def get_player_model(lazy=False):
    """
    Returns the Player model that is active in this project.
    """
    
    try:
        app_label, model_name = PLAYER_MODEL.split('.')
    except ValueError:
        raise ImproperlyConfigured("PLAYER_MODEL must be of the form 'app_label.model_name'")
    if not lazy:
        player_model = get_model(app_label, model_name)
        if player_model is None and IS_PLAYER_REQUIRED:
            print(player_model)
            raise ImproperlyConfigured("PLAYER_MODEL refers to model '{0}' that has not been installed if you don't want to use it, set settings.IS_PLAYER_REQUIRED variable at False ".format(PLAYER_MODEL))
        return player_model
    else:
        return PLAYER_MODEL

GAMESESSION_MODEL = getattr(settings,'GAMESESSION_MODEL','game.Gamesession')
IS_GAMESESSION_REQUIRED = getattr(settings,'IS_GAMESESSION_REQUIRED',True)

def get_gamesession_model(lazy=False):
    """
    Returns the Game Session model that is active in this project.
    """
    
    try:
        app_label, model_name = GAMESESSION_MODEL.split('.')
    except ValueError:
        raise ImproperlyConfigured("GAMESESSION_MODEL must be of the form 'app_label.model_name'")
    if not lazy:
        gamesession_model = get_model(app_label, model_name)
        if gamesession_model is None and IS_GAMESESSION_REQUIRED:
            raise ImproperlyConfigured("GAMESESSION_MODEL refers to model '%s' that has not been installed if you don't want to use it, set settings.IS_GAMESESSION_REQUIRED variable at False " % GAMESESSION_MODEL)
        return gamesession_model
    else:
        return GAMESESSION_MODEL