# -*- coding: utf-8 -*-
""" Frankhood >  apps.facebook.game.managers by Frankhood
    @author: Frankhood Business Solutions
    @author: Gaetano D'Onghia
"""
from __future__ import absolute_import, print_function, unicode_literals

import datetime
import logging
import six

from django.db import models
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible

from fhcore.apps.facebook_v2.settings import DAILY_GAME_ATTEMPTS
from fhcore.utils.backend import convert_date_to_datetime

from .settings import FB_PLAYER_TYPE, SITE_PLAYER_TYPE

logger = logging.getLogger('project')

class PlayerManager(models.Manager):
    
        
    def get_or_create_from_facebook(self, data, user=None):
        try:
            return (self.get(id_attribute=data.get('id'), type=FB_PLAYER_TYPE,),False)
        except self.model.DoesNotExist:
            return (self.create_from_facebook(data, user), True)
    
    def get_from_facebook(self, data, user=None):
        try:
            return self.get(id_attribute=data.get('id'), type=FB_PLAYER_TYPE,)
        except self.model.DoesNotExist:
            return None
    
    
    def create_from_facebook(self, data, user=None):
        try:
            _full_name = "%s %s" % (data.get('first_name', ''), data.get('last_name', ''))
            # logger.info("utente facebook : %s" %(_full_name,))
        except Exception as ex:
            # logger.error("errore inatteso con nome e cognome dell'utente facebook")
            _full_name = ""
        _birthdate = data.get('birthdate', None)
        if _birthdate:
            if not isinstance(_birthdate, datetime.date):
                if isinstance(_birthdate, six.string_types):
                    _birthdate = datetime.datetime.strptime(_birthdate, '%m/%d/%Y')
                if isinstance(_birthdate, datetime.date):
                    _birthdate = datetime.date(_birthdate.year,_birthdate.month,_birthdate.day)
        return self.create(
            username=data.get('username', _full_name),
            first_name = data.get('first_name', ''),
            last_name = data.get('last_name', ''),
            email=data.get('email', ""),
            birthdate=_birthdate,#datetime.datetime.strptime(data.get('birthday'), '%m/%d/%Y') if data.get('birthday') else None,
            type=FB_PLAYER_TYPE,
            id_attribute=data.get('id'),
            user=user,
            extra_data=data,  # json.dumps(data),
        )
            
    def create_from_site(self, user):
        raise NotImplementedError()
    
    def get_fb_users(self):
        queryset = self.get_query_set()
        return queryset.filter(type=FB_PLAYER_TYPE)
    
    def get_site_users(self):
        queryset = self.get_query_set()
        return queryset.filter(type=SITE_PLAYER_TYPE)
    
    
    
class GameSessionManager(models.Manager):

    def init_game(self, player, **kwargs):
        game_session_obj = self.model.objects.create(player=player)  # (**query_kw).save()
        player.update_attempts()
        return game_session_obj

    def get_player_games(self, player):
        return self.get_query_set().filter(player=player)
    
    def get_today_games(self, player):
        _today = timezone.now().date()
        _tomorrow = _today + datetime.timedelta(1)
        _today_datetime = convert_date_to_datetime(_today)
        _tomorrow_datetime = convert_date_to_datetime(_tomorrow)
        return (
            self.get_player_games(player).
            filter(created__range=(_today_datetime, _tomorrow_datetime)).
            order_by('-created')
        )
        
    def get_game(self, player):
        today_game_sessions = self.get_today_games(player)
        if today_game_sessions.count():
            return today_game_sessions[0:1].get()
        return None

    def get_or_init_game(self, player):
        gs = self.get_game(player)
        if gs:
            return (gs, False)
        else:
            if self.can_init_game(player):
                return (self.init_game(player), True)
            else:
                return (None, False)

    def can_init_game(self, player):
        """
        Puoi creare una game session se ne hai già create meno di 
        DAILY_GAME_ATTEMPTS e se nn ce n'è una nn completata
        """
        total_player_games = self.get_player_games(player)
        #logger.info('total_player_games: {0}'.format(total_player_games))
        #logger.info('player.get_total_available_attempts(): {0}'.format(player.get_total_available_attempts()))
        #logger.info('total_player_games.filter(is_completed=False).count(): {0}'.format(total_player_games.filter(is_completed=False).count()))
        if ((total_player_games.count() < player.get_total_available_attempts())
                and (not total_player_games.filter(is_completed=False).count())):
            today_player_games = self.get_today_games(player)
            #logger.info('today_player_games: {0}'.format(today_player_games))
            #logger.info('player.get_daily_available_attempts(): {0}'.format(player.get_daily_available_attempts()))
            #logger.info('today_player_games.filter(is_completed=False).count(): {0}'.format(today_player_games.filter(is_completed=False).count()))
        
            return (today_player_games.count() < player.get_daily_available_attempts() 
                    and (not today_player_games.filter(is_completed=False).count()))
        return False
        
