# -*- coding: utf-8 -*-
""" Frankhood >  fhcore.apps.facebook.models
    @author: Frankhood Business Solutions
    @author: Gaetano D'Onghia
"""
from __future__ import absolute_import, print_function, unicode_literals
from django.utils.encoding import python_2_unicode_compatible

import datetime
import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.template.defaultfilters import safe, date, time as django_time, date as django_date
from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from fhcore.models.base import BaseModel
from fhcore.apps.facebook_v2 import get_player_model, get_gamesession_model
from fhcore.apps.facebook_v2.managers import PlayerManager, GameSessionManager
from fhcore.apps.facebook_v2.settings import (ORIGIN_CHOICES, FB_ORIGIN, 
                                              PLAYER_CHOICES, DAILY_GAME_ATTEMPTS, 
                                              FB_PLAYER_TYPE, TOTAL_GAME_ATTEMPTS)

logger = logging.getLogger(__name__)


try:
    # Django >= 1.7
    import json
except ImportError:
    # Django <= 1.6 backwards compatibility
    from django.utils import simplejson as json

try:
    from django_extensions.db.fields.json import JSONField
    from django_extensions.db.fields import UUIDField

except ImportError:
    raise ImproperlyConfigured("FH Facebookv2 Models require the 'django_extensions' cots module")


from django.conf import settings
try:
    # Django >= 1.7
    #from django.contrib.auth import get_user_model
    User = settings.AUTH_USER_MODEL
except ImportError:
    # Django < 1.7
    from django.contrib.auth.models import User
GamePlayer = get_player_model(lazy=True)

        



class AccountBaseMixin(models.Model):
    username = models.CharField(_("Username"), max_length=255, blank=True, null=True)
    first_name = models.CharField(_("Nome"), max_length=255, blank=True, null=True)
    last_name = models.CharField(_("Cognome"), max_length=255, blank=True, null=True)
    email = models.EmailField(_('Email'), blank=True)
    personal_data = models.BooleanField(_('Acconsente al trattamento dei dati a fini informativi e promozionali'), default=False)

    class Meta(BaseModel.Meta):
        abstract = True

    @property
    def get_first_name(self):
        return self.first_name    

    @property
    def get_last_name(self):
        return self.last_name

    @property
    def get_full_name(self):
        return "{0} {1}".format(self.get_first_name ,self.get_last_name)

    @property
    def get_user_name(self):
        return self.username or self.get_full_name
   
class AccountAdvancedMixin(AccountBaseMixin):
    gender = models.CharField(_('Sesso'), max_length=10, blank=True,null=True)
    phone = models.CharField(_("Telefono"), max_length=18, blank=True, null=True)
    birthdate = models.DateField(_('Data di nascita'), blank=True, null=True)
    avatar = models.ImageField(_("Avatar"), max_length=255, 
                 upload_to='uploads/game/players/avatars',
                 blank=True, null=True)

    class Meta(BaseModel.Meta):
        abstract = True

  
class Player(AccountAdvancedMixin, BaseModel):
    
    objects = PlayerManager()
    
    type = models.CharField(_("Tipologia Player"), max_length=155,
                            choices=PLAYER_CHOICES)
    
    id_attribute = models.CharField(_("Identificativo Player"), max_length=155,db_index=True)
    # lo user può non esserci per esempio se si tratta di un facebook user
    user = models.ForeignKey(User, verbose_name=_("User"), related_name= "%(class)ss", 
                             blank=True, null=True)
    
    extra_data = JSONField()
    origin = models.CharField(_("Registrazione da: "),
                              choices=ORIGIN_CHOICES,
                              max_length=15, default=FB_ORIGIN)
    #LOGICA DI GIOCO
    total_available_attempts = models.IntegerField(_("Giocate Disponibili"), 
                                             blank=True, null=True, 
                                             default=TOTAL_GAME_ATTEMPTS )
    daily_available_attempts = models.IntegerField(_("Giocate Giornaliere Disponibili"), 
                                             blank=True, null=True, 
                                             default=DAILY_GAME_ATTEMPTS )
    used_attempts = models.IntegerField(_("Giocate Utilizzate"), default=0)
    
    
    
    class Meta(BaseModel.Meta):
        abstract = True
        ordering = ['-created',] #ordinati x created : LIFO
        verbose_name= _('Player')
        verbose_name_plural= _('Players') 
        unique_together = ('id_attribute', 'type')
        
    def __unicode__(self):
        try:
            return self.get_full_name
        except:
            return "{0}".format(self.player_data,)

    def is_registered(self):
        return True
        """if all((self.first_name, self.last_name, self.email, self.birthdate,)):
            return True
        return False"""

    @cached_property
    def get_avatar_url(self):
        return "https://graph.facebook.com/{0}/picture?width={1}&height={2}".format(
                    self.id_attribute, 57, 57)

    @cached_property
    def get_mobile_avatar_url(self):
        return "https://graph.facebook.com/{0}/picture?width={1}&height={2}".format(
                    self.id_attribute, 84, 84)

    @property
    def facebook_id(self):
        if self.type == FB_PLAYER_TYPE:
            return self.id_attribute
        return None
    
    @property
    def get_first_name(self):
        datadict = self.extra_data#json.loads(self.extra_data)
        if self.user:
            first_name = self.first_name or datadict.get('first_name',self.user.first_name)
        else:
            first_name = self.first_name or datadict.get('first_name','')
        return first_name    

    @property
    def get_last_name(self):
        datadict = self.extra_data#json.loads(self.extra_data)
        if self.user:
            last_name = self.last_name or datadict.get('last_name',self.user.last_name)
        else:
            last_name = self.last_name or datadict.get('last_name','')
        return last_name

    @property
    def get_full_name(self):
        return "{0} {1}".format(self.get_first_name ,self.get_last_name)

    @property
    def get_email(self):
        if self.email:
            return self.email
        elif self.user and self.user.email:
            return self.user.email
        return None

    @property
    def player_data(self):
        if self.username:
            return self.username
        else:
            if self.user:
                return self.user.get_full_name() or self.user.username
        return self.get_email #self.email or ''

    ##################################################
    #######  API x GET/CREATE  Sessioni di Gioco   ###
    ##################################################

    def get_total_coins(self):
        Game = get_gamesession_model()
        player_games = Game.objects.get_player_games(self)
        today_coins = TOTAL_GAME_ATTEMPTS - player_games.count()
        return today_coins 
       
    def get_today_coins(self):
        Game = get_gamesession_model()
        today_games = Game.objects.get_today_games(self)
        today_coins = DAILY_GAME_ATTEMPTS - today_games.count()
        return today_coins
    
    def can_create_game_sessions(self):
        Game = get_gamesession_model()
        return Game.objects.can_init_game(self)

    def get_or_create_game_session(self):
        Game = get_gamesession_model()
        return Game.objects.get_or_init_game(self)

    def get_game_session(self):
        Game = get_gamesession_model()
        return Game.objects.get_game(self)

    def create_game_session(self):
        Game = get_gamesession_model()
        return Game.objects.init_game(self)        

    
    @property
    def remaining_attempts(self):
        """solo per i totali ma diventa giornaliero se used_attempts viene azzerato ogni giorno"""
        return self.get_total_available_attempts() - self.used_attempts
    
    def update_attempts(self):
        self.used_attempts += 1
        self.save()
    update_attempts.alters_data=True

    def get_total_available_attempts(self):
        return self.total_available_attempts or TOTAL_GAME_ATTEMPTS

    def get_daily_available_attempts(self):
        return self.daily_available_attempts or DAILY_GAME_ATTEMPTS


class GameSession(BaseModel):
    
    objects = GameSessionManager()
    #played_games = GamePlayedManager()

    uuid = UUIDField(_("uuid"), max_length=100, unique=True, editable=False)
    player = models.ForeignKey(GamePlayer,
                               # null=True, blank=True,
                               related_name='games',
                               verbose_name=_("Giocatore"))

    is_completed = models.BooleanField(_("Gioco terminato"), default=False)

    class Meta(BaseModel.Meta):
        abstract=True
        ordering = ['-created', ]  # ordinati x created : LIFO
        verbose_name = _('Game')
        verbose_name_plural = _('Games')    

    @property
    def is_expired(self):
        """Gioco scaduto"""
        expired = self.modified + datetime.timedelta(hours=24) < timezone.now()
        return expired

    @property
    def is_playable(self):
        """Il gioco è ancora attivo, cioè non completato e non scaduto"""
        return (not self.is_completed) and (not self.is_expired)
    
    @property
    def is_active(self):
        return self.is_playable #or self.is_now_able_to_win
    
    def __unicode__(self):
        if self.player:
            return u"Sessione di Gioco %s per il giocatore %s in data %s" % (self.uuid, self.player, django_date(self.created, "d/m/Y H:m"))
        else:
            return u"Sessione di Gioco %s anonima in data %s" % (self.uuid, django_date(self.created, "d/m/Y H:m"))
