# -*- coding: utf-8 -*-
""" fHCoreLibs.apps.facebook.views.game by Frankhood
    @author: Frankhood Business Solutions
    @author: Gaetano D'Onghia
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging

from django.contrib import messages
from django.utils.encoding import smart_str
from django.views.generic.base import TemplateResponseMixin, View

from fhcore.apps.facebook.views.exceptions import ActionNotAllowed
from .mixins import FBContextMixin, FBUserMixin, \
    FBPlayerMixin
from fhcore.views.base import HeadResponseMixin, JSONResponseMixin

from .mixins import FBPlayerSessionMixin

logger = logging.getLogger('project')


class FBUserView(TemplateResponseMixin, FBContextMixin, FBUserMixin, HeadResponseMixin, View):

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)


class FBAnonymousUserView(FBUserView):
    is_oauth_required = False


class FBAuthenticatedUserView(FBUserView):
    is_oauth_required = True


####PLAYER REQUIRED
class FBPlayerView(TemplateResponseMixin, FBContextMixin, FBPlayerMixin, HeadResponseMixin, View):

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)


####PLAYER REQUIRED
class FBPlayerSessionView(TemplateResponseMixin, FBContextMixin, FBPlayerSessionMixin, HeadResponseMixin, View):

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)


class FBAnonymousPlayerView(FBPlayerSessionView):
    is_oauth_required = False
    is_player_required = False


# DA RIVEDERE
class BaseAPIView(JSONResponseMixin, FBUserView):
    """
    a generic (kinda :-DD) REST view
    non ancora utilizzata
    """

    model = None
    add_form = None
    slug_name = 'pk'

    def get_slug_value(self):
        val = self.kwargs.get(self.slug_name, None) \
             or self.request.GET.get(self.slug_name, None)
        return val

    def get_object(self):
        try:
            obj_lookup = {self.slug_name: self.get_slug_value()}
            obj = self.model.objects.select_related().get(**obj_lookup)
        except self.model.DoesNotExist, ex:
            logger.error(ex)
            obj = None
        return obj

    def get_context_data(self, status, msg='', obj=None, **kwargs):
        """ @return <dict> context
            {
                'status': <string> ('ERROR' | 'WARNING' | 'SUCCESS')
                'message': <string>
                [, 'obj_name': <dict> ]
                [, **kwargs]
            }
        """
        context = {}
        context['status'] = messages.DEFAULT_TAGS[status]
        context['message'] = smart_str(msg)
        if obj:
            obj_name = self.model._meta.object_name.lower()
            try:
                context[obj_name] = obj.get_as_dict()
            except Exception, ex:
                logger.warning("Unable to convert object to dict: %s" % ex)
                context[obj_name] = obj
        if len(kwargs):
            context.update(kwargs)
        return context

    # -- actions
    def _save_data(self):
        raise NotImplementedError

    def _fetch_data(self):
        try:
            if not self.object:
                raise ActionNotAllowed(_(u"Attenzione! la risorsa richiesta non esiste"
                                          " o non hai i permessi per accedervi"))
            status = messages.SUCCESS
            context_data = self.get_context_data(status, obj=self.object)
        except ActionNotAllowed, ex:
            status = messages.WARNING
            context_data = self.get_context_data(status, ex)
            logger.warning(ex)
        return context_data

    # -- request methods
    def get(self, request, *args, **kwargs):
        """ retrieve an object """
        self.object = self.get_object()
        context = self._fetch_data()
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        """ save a new object """
        self.object = self.get_object()
        context = self._save_data()
        return self.render_to_response(context)
