# -*- coding: utf-8 -*-
""" Frankhood : fh_statistics.views.base by Frankhood
    @author : Frankhood Business Solution
"""
import datetime
import logging
        
from django.contrib.admin.views.decorators import staff_member_required
from django.template.defaultfilters import time as django_time, date as django_date
from django.template.defaultfilters import safe
from django.utils.decorators import method_decorator
from django.utils.functional import cached_property
from django.views.generic.base import TemplateView


from ..models import StatisticCache
from fhcore.utils.querys import dictfetchall


logger = logging.getLogger(__name__)

try:
    # Django >= 1.7
    import json
except ImportError:
    # Django <= 1.6 backwards compatibility
    from django.utils import simplejson as json
    
########################################
# Base
########################################
class BaseStatsView(TemplateView):
    """
    Vista base dalla quale una vista di Statistica deve ereditare
    """
    template_name = None 
    
    reverse_name = ""
    # reverse_name rappresenta un identificativo che deve essere uguale al reverse_name nel file urls.py
    # in pratica ci sarà una corrispondenza 1 a 1 tra vista e url
    
    view_label = ""
    # view_label è un etichetta che serve in fase visualizzazione per l'utente e quindi deve essere abbastanza esplicativo per indicare lo scopo di quela vista (o statistica) 
    
    ajax_reverse_name = ""
    
    @method_decorator(staff_member_required)
    def dispatch(self, request, *args, **kwargs):
        return super(BaseStatsView, self).dispatch(request, *args, **kwargs)

    def get_label(self):
        return self.view_label  # (IL TITOLO DEL GRAFICO)

    def get_legenda(self):
        raise NotImplementedError
        
    def get_queryset(self):
        raise NotImplementedError
    
    def get_total_count(self, *args, **kwargs):
        queryset = self.get_queryset()
        return queryset.count()
    
    def get_regroup_kwargs(self, *args, **kwargs):
        return kwargs

    def regroup_queryset(self, *args, **kwargs):
        queryset = kwargs.get('queryset', self.get_queryset())
        return queryset
    
    def get_data_array(self, **kwargs):
        queryset = self.get_queryset()
        regroup_kwargs = self.get_regroup_kwargs(**({'queryset':queryset}))
        data_array = self.regroup_queryset(**(regroup_kwargs))
        return data_array
        
    def get_stats(self, **kwargs):
        chart_legenda_array = self.get_legenda()
        chart_data_array = self.get_data_array(**kwargs)
        chart_array = []
        chart_array.extend([chart_legenda_array])
        chart_array.extend(chart_data_array)
        stats = {'label': self.get_label(),
                'total_count': self.get_total_count()
                 }
        if not self.request.is_ajax():
            stats.update({'json_legenda_array': json.dumps(chart_legenda_array)})
            stats.update({'json_data_array': json.dumps(chart_data_array)})
            stats.update({'json_array': json.dumps(chart_array)})
        else:
            stats.update({'json_legenda_array': chart_legenda_array})
            stats.update({'json_data_array': chart_data_array})
            stats.update({'json_array': chart_array})
        return stats
    
    def get_reverse_url_kwargs(self):
        return {}
    
    def get_context_data(self, **kwargs):
        stats = self.get_stats(**kwargs)
        return {
            'params': kwargs,
            'stats' : stats,
            'url_reverse_name' : self.ajax_reverse_name,
            'url_reverse_kwargs' : json.dumps(self.get_reverse_url_kwargs())
        }


class DateGroupingMixin(object):
    
    DEFAULT_DAY_START = 1
    DEFAULT_MONTH_START = 1
    DEFAULT_YEAR_START = 2014
    
    DEFAULT_DAY_END = 31
    DEFAULT_MONTH_END = 12
    DEFAULT_YEAR_END = 2018
    
    def get_legenda(self):
        return ['Day', 'Games']
    
    def get_begin_date_from_form(self):
        begin_year = int(self.request.GET.get('begin_y', self.DEFAULT_YEAR_START))
        begin_month = int(self.request.GET.get('begin_m', self.DEFAULT_MONTH_START))
        begin_day = int(self.request.GET.get('begin_d', self.DEFAULT_DAY_START))
        return (begin_year, begin_month, begin_day)    

    def get_end_date_from_form(self):
        end_year = int(self.request.GET.get('end_y', self.DEFAULT_YEAR_END))
        end_month = int(self.request.GET.get('end_m', self.DEFAULT_MONTH_END))
        end_day = int(self.request.GET.get('end_d', self.DEFAULT_DAY_END))
        return (end_year, end_month, end_day)            
    
    def get_range_days_from_form(self):
        try:
            return int(self.request.GET.get('range_days', None))
        except:
            return None

    def get_initial_date_from_range(self):
        try:
            return int(self.request.GET.get('begin_offset', None))
        except:
            return None

    def get_begin_date(self, *args, **kwargs):
        """Begin date Real, non modificabile perchè da l'inizio utile per il filtro delle statistiche """
        try:
            (begin_year, begin_month, begin_day) = self.get_begin_date_from_form()
            begin_date = datetime.datetime(begin_year,
                                            begin_month,
                                            begin_day)
            #logger.info("begin_date from form : {0}".format(begin_date))
        except ValueError:
            begin_date = datetime.datetime(self.DEFAULT_YEAR_START,
                                            self.DEFAULT_MONTH_START,
                                            self.DEFAULT_DAY_START)
        begin_offset = self.get_initial_date_from_range()
        #logger.info("begin_offset : {0}".format(begin_offset))
        if begin_offset:
            begin_date = begin_date + datetime.timedelta(begin_offset)
        #logger.info("begin_date : {0}".format(begin_date))
        return kwargs.get('begin_date', begin_date)
    
    def get_end_date(self, *args, **kwargs):
        try:
            (end_year, end_month, end_day) = self.get_end_date_from_form()
            finish_date = datetime.datetime(end_year, end_month, end_day) + datetime.timedelta(days=1)
        except ValueError:
            finish_date = datetime.datetime(self.DEFAULT_YEAR_END,
                                             self.DEFAULT_MONTH_END,
                                             self.DEFAULT_DAY_END) + datetime.timedelta(days=1)
        finish_date = kwargs.get('end_date', finish_date)
        #logger.info("finish_date : {0}".format(finish_date))
        today_end_date = datetime.datetime.today()# + datetime.timedelta(1)
        #logger.info("today_end_date : {0}".format(today_end_date))
        finish_date = finish_date if finish_date <= today_end_date else today_end_date
        #logger.info("finish_date : {0}".format(finish_date))
        return finish_date #+ datetime.timedelta(days=1)

    def get_date_range(self, *args, **kwargs):
        # se mando un date range non uso l'end_date
        date_range = self.get_range_days_from_form()
        #logger.info("date_range from form : {0}".format(date_range))
        if date_range:
            return datetime.timedelta(date_range)
        else:
            range_date = self.get_end_date() - self.get_begin_date()
            """
            try:
                (end_year, end_month, end_day) = self.get_end_date_from_form()
                finish_date = datetime.datetime(end_year, end_month, end_day)
                range_date = finish_date - self.get_begin_date() + datetime.timedelta(days=1)
            except ValueError:
                range_date = self.get_end_date() - self.get_begin_date()
                """
            #logger.info("begin date : {0}".format(self.get_begin_date()))
            #logger.info("end date : {0}".format(self.get_end_date()))
            #logger.info("date_range from form : {0}".format(range_date))
            return range_date

    def get_filter_kwargs(self):
        kw = {}
        return kw
    
    def get_regroup_kwargs(self, **kwargs):
        begin_date = self.get_begin_date()
        daterange = self.get_date_range()
        filter_kwargs = self.get_filter_kwargs()
        kwargs.update({'key_string':'created',
                        'begin_date':begin_date,
                        'daterange':daterange,
                        'filter_kwargs':filter_kwargs})
        return kwargs
    
    def init_global_variables(self):
        # da ridefinire :
        # inizializzo variabili di classe da utilizzare nel metodo "regroup_daily_queryset"
        pass

    def regroup_daily_queryset(self, daily_qs, **kwargs):
        return [daily_qs.count()]
       
    def regroup_queryset(self, **kwargs):
        import time
        queryset = kwargs.get('queryset', [])
        key_string = kwargs.get('key_string', None)
        begin_date = kwargs.get('begin_date', self.get_begin_date())
        daterange = kwargs.get('daterange', self.get_date_range())
        filter_kwargs = kwargs.get('filter_kwargs', {})
        data = []
        self.init_global_variables()
        for date_item in range(daterange.days):
            iterdate = begin_date + datetime.timedelta(days=date_item)
            queryset_filters = filter_kwargs
            queryset_filters.update({"%s__year" % key_string : iterdate.year,
                                     "%s__month" % key_string : iterdate.month,
                                     "%s__day" % key_string : iterdate.day,
                                    })
            # faccio il regroup x avere utenti distinti
            daily_qs = list()
            daily_qs = queryset.filter(**(queryset_filters))
            daily_data = [self.format_date(iterdate)]
            daily_data.extend(self.regroup_daily_queryset(daily_qs, **({'current_date':iterdate})))
            self.daily_array = daily_data
            data.append(daily_data)    
        return data
    
    def format_date(self, _date):
        return django_date(_date, "m/d/Y")
    
    def get_stats(self, **kwargs):
        stats = super(DateGroupingMixin, self).get_stats(**kwargs)
        stats.update({'days':range(1, 32),
                      'months': range(1, 13),
                      'years':range(self.DEFAULT_YEAR_START, self.DEFAULT_YEAR_END + 1)})
        days_range = self.get_date_range().days
        stats.update({
              'begin_dates':tuple(self.get_begin_date_from_form()),
              'end_dates':tuple(self.get_end_date_from_form()),
              # 'daterange': daterange,
              'days_range':days_range + 1, # + 1;
              'begin_ajax_dates': tuple(self.get_begin_date().timetuple()),
              'end_ajax_dates': tuple(self.get_end_date().timetuple()),
        })
        #json_dates=[str(self.format_date(_date)) for _date in (self.get_begin_date() +datetime.timedelta(days=_days) for _days in range(days_range+1))]
        #stats.update({'json_dates':json.dumps(json_dates)}),
        #logger.info("Days Range JSON : %s" % json.dumps(json_dates))
        #logger.info("Days Range : %s" % json_dates)
        #chart_dates_array = [x[0] for x in chart_data_array]
        #stats.update({'json_dates': json.dumps(chart_dates_array)})
        return stats
    
    
    def get_context_data(self, *args, **kwargs):
        context = super(DateGroupingMixin, self).get_context_data(*args, **kwargs)
        today = datetime.datetime.today().date()
        context.update({
            "today_y":today.year,
            "today_m":today.month,
            "today_d":today.day
        })
        return context
        

class DateBasedStatsView(DateGroupingMixin, BaseStatsView):
    reverse_name = ""
    view_label = ""   

class DailyAdvancedStatsView(DateBasedStatsView):
    template_name = None 
    reverse_name = ""
    view_label = ""

    def get_context_data(self,*args, **kwargs):
        context = super(DailyAdvancedStatsView, self).get_context_data(*args, **kwargs)
        logger.info("Context : {0}".format(context))
        return context

class BaseEmptyView(DailyAdvancedStatsView):

    def get_total_count(self):
        return "" 
    
    def get_data_array(self, **kwargs):
        legenda_array = self.get_legenda()
        data_array = []
        begin_date = kwargs.get('begin_date', self.get_begin_date())
        daterange = kwargs.get('daterange', self.get_date_range())
        for date_item in range(daterange.days):
            daily_result_array = []
            daily_date = begin_date + datetime.timedelta(days=date_item)
            daily_formatted_date = self.format_date(daily_date)
            daily_result_array.append(daily_formatted_date)
            [daily_result_array.append(0) for item in legenda_array[:-1]]
            data_array.append(daily_result_array)
        return data_array

class BaseCachedEmptyView(BaseEmptyView):
    
    cache_namespace = None
    
    @cached_property
    def cached_value(self):
        if not self.cache_namespace:
            return None, None, 0
        start = datetime.date(*self.get_begin_date_from_form())
        end = datetime.date(*self.get_end_date_from_form())
        logger.info('start : %s' %( start, ))
        logger.info('end : %s' %( end, ))
        i = start
        start_cache = None
        end_cache = None
        while i <= end:
            _cache = None
            try:
                _cache = StatisticCache.objects.get(key=str(i), namespace=self.cache_namespace)
                start_cache = start
                end_cache = i
                i += datetime.timedelta(days=1)
            except Exception, ex:
                break
        if all((start_cache, end_cache)):
            diff = (end_cache - start_cache).days +1#// + 1
        else:
            diff = 0
        logger.info('start_cache : %s' %( start_cache, ))
        logger.info('end_cache : %s' %( end_cache, ))
        logger.info('diff : %s' %( diff, ))
        return start_cache, end_cache, diff
    
    def get_begin_date(self, *args, **kwargs):
        d = super(BaseCachedEmptyView, self).get_begin_date(*args, **kwargs)
        a, b, c = self.cached_value
        if b and d <= datetime.datetime.combine(b, datetime.time()):
            return datetime.datetime.combine(b + datetime.timedelta(days=1), datetime.time())
        return d
    
    def get_end_date(self, *args, **kwargs):
        d = super(BaseCachedEmptyView, self).get_end_date(*args, **kwargs)
        a, b, c = self.cached_value
        if b and d <= datetime.datetime.combine(b, datetime.time()):
            return datetime.datetime.combine(b + datetime.timedelta(days=1), datetime.time())
        return d
    
    def get_context_data(self, *args, **kwargs):
        context = super(BaseCachedEmptyView, self).get_context_data(*args, **kwargs)
        cached_start, cached_end, cached_range = self.cached_value
        context.update({'cached': {
            'cached_begin_dates':cached_start,
            'cached_range':cached_range,
        }})
        return context
    
class RawQueryStatMixin(object):    
    alias_name = "result"
    cache_namespace = None
    
    def get_raw_querys(self):
        query1 = """
                insert here your query
                """
        query2 = """
                insert here your query
                """
        return [query1, query2]    
    
    def format_query(self, query, format_tuple):
        # logger.info("format_tuple: ", format_tuple)
        formatted_query = query.format(*format_tuple)
        return formatted_query
    
    def get_alias_name(self):
        return self.alias_name
    
    def init_global_variables(self):
        # inserisci qui le variabili globali da incrementare
        pass
    
    def do_raw_query(self, query):
        from django.db import connection, transaction
        cursor = connection.cursor()
        cursor.execute(query)
        row = dictfetchall(cursor)
        # logger.info("raw result {0}".format(row[0]))
        result = row[0][self.get_alias_name()]
        try:
            # close cursor
            cursor.close()
        except:
            pass
        return result

    def do_raw_queries(self, *args, **kwargs):
        raw_querys = self.get_raw_querys()
        daily_result_array = []
        for raw_query in raw_querys:
            formatted_query = self.format_query(raw_query,*args, **kwargs)
            result = self.do_raw_query(formatted_query)
            daily_result_array.append(result)
        return daily_result_array

    def is_cachable(self, *args, **kwargs):
        '''
            Default caching policy: not today
        '''
        daily_date = kwargs.get('daily_date', None)
        if daily_date:
            try:
                daily_date = daily_date.date()
            except:
                raise
        today = datetime.date.today()
        if self.cache_namespace and  daily_date < today:
            return True
        return False

    def get_data_array(self, **kwargs):
        data_array = []
        begin_date = kwargs.get('begin_date', self.get_begin_date())
        daterange = kwargs.get('daterange', self.get_date_range())
        self.init_global_variables()
        for date_item in range(daterange.days):
            daily_date = begin_date + datetime.timedelta(days=date_item)
            daily_formatted_date = self.format_date(daily_date)
            daily_query_date = django_date(daily_date, "Y-m-d")
            cache_key = daily_query_date
            daily_result_array = None
            # Controllo la cache
            if self.is_cachable(daily_date=daily_date):
                try:
                    _cache = StatisticCache.objects.get(key=str(cache_key), namespace=self.cache_namespace)
                    daily_result_array = _cache.value
                except:
                    daily_result_array = None
            if not daily_result_array:
                daily_result_array = []
                daily_result_array.append(daily_formatted_date)
                daily_result_array.extend(self.do_raw_queries([str(daily_query_date),]))
                if self.is_cachable(daily_date=daily_date):
                    try:
                        _cache, created = StatisticCache.objects.get_or_create(key=str(cache_key), namespace=self.cache_namespace)
                        _cache.value = daily_result_array
                        _cache.save()
                    except:
                        pass
            data_array.append(daily_result_array)
        return data_array
