# -*- coding: utf-8 -*-
""" FHCore Libs  >   
    @author: Frankhood Business Solutions <info@frankhood.it>
    @author: Gaetano D'Onghia <g.donghia@frankhood.it>
"""
from __future__ import absolute_import, print_function, unicode_literals
import logging

from django.utils.translation import ugettext, ugettext_lazy as _
from django.http.response import HttpResponseRedirectBase

from fhcore.utils.backend import get_subclasses
from fhcore.views.base import JSONStructuredResponseMixin

logger = logging.getLogger(__name__)

try:
    # Django >= 1.7
    import json
except ImportError:
    # Django <= 1.6 backwards compatibility
    from django.utils import simplejson as json

class JSONRedirectMiddleware(JSONStructuredResponseMixin):
    """
    Middleware that transform an HttpResponseRedirect and HttpResponsePermanentRedirect
    in a JSON packet.
    
    Add this row to your settings.MIDDLEWARE_CLASSES:
    
        'fhcore.middleware.hijax.JSONRedirectMiddleware',

    """
    
    def get_content_type(self, **kwargs):
        return 'text/html'

    def get_check_is_redirect(self, **kwargs):
        #"""
        #Sempre True xke se ci passi vuol dire che è una redirect
        #"""
        return True
    
    def process_response(self, request, response):
        if response.__class__ in get_subclasses(HttpResponseRedirectBase):
            #print("JSONRedirectMiddleware : E' una Redirect")
            if request.is_ajax():
                return self.get_json_response(self.convert_context_to_json(
                                                {'redirect_url':response['Location']}
                                                ),**({'status':401}))
        return response