# -*- coding: utf-8 -*-
""" fhcore.templatetags.fh_tags.py by Frankhood
    @author: Frankhood Business Solutions
"""
import re
import logging
from django import template
#from django.template.loaders.app_directories import load_template_source
from django.template.base import TemplateSyntaxError
 
register = template.Library()
logger = logging.getLogger('project')
'''
def do_include_raw(parser, token):
    """ 
    Performs a template include without parsing the context, just dumps the template in.
    @author : http://djangosnippets.org/snippets/1684/
    """
    bits = token.split_contents()
    if len(bits) != 2:
        raise TemplateSyntaxError, "%r tag takes one argument: the name of the template to be included" % bits[0]
 
    template_name = bits[1]
    if template_name[0] in ('"', "'") and template_name[-1] == template_name[0]:
        template_name = template_name[1:-1]
 
    source, path = load_template_source(template_name)
 
    return template.TextNode(source)

register.tag("include_raw", do_include_raw)

'''

@register.tag
def annotate_form_field(parser, token):
    """
    Set an attribute on a form field with the widget type

    This means templates can use the widget type to render things differently
    if they want to.  Django doesn't make this available by default.
    
    Usage:
    {% annotate_form_field my_field %}
    
    {% if my_field.widget_type == 'CheckboxInput' %}
        {% include 'partials/form_field_checkbox.html' with field=my_field %}
    {% elif my_field.widget_type == 'RadioInput' %}
        {% include 'partials/form_field_radio.html' with field=my_field %}
    {% else %}
        {% include 'partials/form_field.html' with field=my_field %}
    {% endif %}
    """
    args = token.split_contents()
    if len(args) < 2:
        raise TemplateSyntaxError(
            "annotate_form_field tag requires a form field to be passed")
    return FormFieldNode(args[1])


class FormFieldNode(template.Node):
    def __init__(self, field_str):
        self.field = template.Variable(field_str)

    def render(self, context):
        field = self.field.resolve(context)
        if hasattr(field, 'field'):
            field.widget_type = field.field.widget.__class__.__name__
        return ''
    
try:
    from classytags.core import Options
    from classytags.core import Tag as TemplateTag
    from classytags.arguments import Argument

    class BuildAbsoluteUri(TemplateTag):
        
        name = 'build_absolute_uri'
        
        options = Options(
            'path', Argument('path', required=True, resolve=True),
            'as', Argument('as_var', required=False, resolve=False),
        )
    
        def render_tag(self, context, path=None, as_var=None):
            request = context['request']
            absolute_uri = request.build_absolute_uri(path)
            if as_var:
                context[as_var] = absolute_uri
            return absolute_uri
    register.tag(BuildAbsoluteUri)
    
except ImportError:
    pass

#From here : http://stackoverflow.com/questions/1070398/how-to-set-a-value-of-a-variable-inside-a-template-code
class SetVarNode(template.Node):
    def __init__(self, new_val, var_name):
        self.new_val = new_val
        self.var_name = var_name
    def render(self, context):
        context[self.var_name] = self.new_val
        return ''


@register.tag
def setvar(parser,token):
    # This version uses a regular expression to parse tag contents.
    try:
        # Splitting by None == splitting by spaces.
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires arguments" % token.contents.split()[0]
    m = re.search(r'(.*?) as (\w+)', arg)
    if not m:
        raise template.TemplateSyntaxError, "%r tag had invalid arguments" % tag_name
    new_val, var_name = m.groups()
    if not (new_val[0] == new_val[-1] and new_val[0] in ('"', "'")):
        raise template.TemplateSyntaxError, "%r tag's argument should be in quotes" % tag_name
    return SetVarNode(new_val[1:-1], var_name)


"""
Decorator to facilitate template tag creation
"""
def easy_tag(func):
    """deal with the repetitive parts of parsing template tags"""
    def inner(parser, token):
        #print token
        try:
            return func(*token.split_contents())
        except TypeError:
            raise template.TemplateSyntaxError('Bad arguments for tag "%s"' % token.split_contents()[0])
    inner.__name__ = func.__name__
    inner.__doc__ = inner.__doc__
    return inner


class AppendGetNode(template.Node):
    """
    takenFrom https://djangosnippets.org/snippets/1627/
    example usage :
    {% for page_num in results.paginator.page_range %}     
    <a href="{% append_to_get p=page_num %}">{{ page_num }}</a>
    {% endfor %}
    """
    
    def __init__(self, dict):
        self.dict_pairs = {}
        for pair in dict.split(','):
            pair = pair.split('=')
            self.dict_pairs[pair[0]] = template.Variable(pair[1])
            
    def render(self, context):
        get = context['request'].GET.copy()
        logger.info('get pre: {0}'.format(get))
        for key in self.dict_pairs:
            get[key] = self.dict_pairs[key].resolve(context)
            logger.info('get[{0}] : {1}'.format(key, get[key]))
        
        path = context['request'].META['PATH_INFO']
        logger.info('path : {0}'.format(path))
        
        #print "&".join(["%s=%s" % (key, value) for (key, value) in get.items() if value])
        logger.info('get post: {0}'.format(get))
        
        if len(get):
            path += "?%s" % "&".join(["%s=%s" % (key, value) for (key, value) in get.items() if (value is not "" and value is not None) ])
        return path

@register.tag()
@easy_tag
def append_to_get(_tag_name, dict):
    return AppendGetNode(dict)