# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from functools import update_wrapper
try:
    import json
except:
    from django.utils import simplejson as json

from django.contrib.admin.models import LogEntry, ADDITION, CHANGE
from django.contrib.contenttypes.models import ContentType
from django.contrib import messages
from django.utils.decorators import classonlymethod
from django.utils.encoding import force_unicode


class AsActionMixin(object):

    short_description = None
    action_name = None

    messages = messages

    @classonlymethod
    def as_action(cls, **initkwargs):
        """
        Main entry point for a request-response process.
        """
        # sanitize keyword arguments
        for key in initkwargs:
            if key in cls.http_method_names:
                raise TypeError(u"You tried to pass in the %s method name as a "
                                u"keyword argument to %s(). Don't do that."
                                % (key, cls.__name__))
            if not hasattr(cls, key):
                raise TypeError(u"%s() received an invalid keyword %r" % (
                    cls.__name__, key))
        initkwargs['action'] = True

        def view(modeladmin, request, queryset, *args, **kwargs):
            self = cls(**initkwargs)
            kwargs.update({'modeladmin': modeladmin, 'queryset': queryset})
            # New in django 1.6
            self.request = request
            self.args = args
            self.kwargs = kwargs
            return self.dispatch(request, *args, **kwargs)

        # take name and docstring from class
        update_wrapper(view, cls, updated=())

        # and possible attributes set by decorators
        # like csrf_exempt from dispatch
        update_wrapper(view, cls.dispatch, assigned=())
        if cls.action_name:
            view.__name__ = cls.action_name
        view.short_description = cls.short_description
        return view

    def admin_log(self, obj, change_message, action_flag=None):
        if action_flag == None:
            action_flag = 99
        LogEntry.objects.log_action(
            user_id=self.request.user.pk,
            content_type_id=ContentType.objects.get_for_model(obj).pk,
            object_id=obj.pk,
            object_repr=force_unicode(obj),
            action_flag=action_flag,
            change_message=change_message
        )
