# -*- coding: utf-8 -*-
""" Frankhood >  fhcore.views.file_upload
    @author: Frankhood Business Solutions
    @author: Gaetano D'Onghia
"""
from __future__ import absolute_import, print_function, unicode_literals
from django.utils.encoding import python_2_unicode_compatible

import logging
import os
import base64

from django import http
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.files.base import ContentFile  
from django.core.exceptions import ImproperlyConfigured
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.files.uploadedfile import UploadedFile
from django.http.response import HttpResponse
from django.template.defaultfilters import slugify
from django.utils.datastructures import MultiValueDictKeyError
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import View
from django.views.generic.detail import SingleObjectMixin

from .base import JSONResponseMixin, JSONFormMixin
from ..exceptions import MaximumFileSizeExceeding, FileNotSend
from ..utils.backend import generate_code

try:
    from sorl.thumbnail.shortcuts import get_thumbnail
except ImportError as ex:
    raise ImproperlyConfigured("Sorl Thumbnail is required for image preview")


logger = logging.getLogger('project')

MEDIA_ROOT = getattr(settings, 'MEDIA_ROOT')
STATIC_ROOT = getattr(settings, 'STATIC_ROOT') #"/home/gaetano/Workspace/SoleSorriso/project/core/static/" #getattr(settings, 'STATIC_ROOT')
UPLOAD_ROOT = os.path.join(MEDIA_ROOT, 'uploads')
            
    
class UploadAttachmentView(JSONResponseMixin, View):
    
    image_dims = "80x80"
    image_quality = 50
    max_file_size = 5 * 2**20 # in KB == 5 MB
    upload_path = os.path.join(UPLOAD_ROOT, 'images')
    image_key = "files[]"
    
    
    #@method_decorator(csrf_protect)
    @method_decorator(csrf_exempt)
    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        return super(UploadAttachmentView, self).dispatch(request, *args, **kwargs)
    
    """
    def get_file_name(self, request):
        post_data = request.POST.copy()
        file = post_data.get(self.image_key,None)
        if not file:
            get_data = request.GET.copy()
            file = get_data.get(self.image_key,None)
        logger.info("File prima del codice di merda: {0}".format(file))
        real_file, ext = os.path.splitext(file)
        file= "%s%s" %( slugify(real_file), ext)
        logger.info("File dopo il codice di merda: {0}".format(file))
        return file
    """
    
    def get_no_collition_filename(self,full_path):
        if not os.path.exists(full_path):
            return full_path
        dir, fname = os.path.split(full_path)
        file, ext = os.path.splitext(fname)
        i = 1
        while True:
            new_file_name = os.path.join(dir,"%s_%s%s" % (file,i,ext))
            if not os.path.exists(new_file_name):
                return new_file_name
            i+=1
            
    def create_file_from_raw_data(self,request,filename):
        raw_file = request.raw_post_data
        file, ext = os.path.splitext(filename)
        uploaded_file = SimpleUploadedFile("%s%s" % (file,ext) , raw_file, "image/%s" % ext[1:])
        return uploaded_file
    
    def save_file(self,request,dest_path,file):
        chunk = request.REQUEST.get('chunk', '0')
        chunks = request.REQUEST.get('chunks', '0')
        #logger.info("dest_path : {0}".format(dest_path))
        f = open(dest_path, ('wb' if chunk == '0' else 'ab'))
        for content in file.chunks():
            f.write(content)
            if int(chunk) + 1 >= int(chunks):
                pass
        f.close()
        
        #min_file_name = os.path.split(dest_path)[1]
        #return min_file_name
        return dest_path
    
    def delete_last_file(self,request):
        try:
            last_file= request.session['last_file_name']
            if last_file:
                deleting_file = os.path.join(MEDIA_ROOT, self.upload_path, last_file)
                os.remove(deleting_file)
                return True
            else:
                return False
        except:
            return False

    def get_uploaded_file(self, request, filename=None):
        try:
            #files = request.FILES[u'files[]'] or request.FILES['file']
            uploaded_file = request.FILES[self.image_key]
        except (MultiValueDictKeyError, KeyError),ex:
            uploaded_file = self.create_file_from_raw_data(request, filename)
        return uploaded_file

    def process_data(self, request):
        #uploading_filename =  self.get_file_name(request)
        #if uploading_filename:
        uploaded_file = self.get_uploaded_file(request)
        if uploaded_file:
            uploading_filename = uploaded_file.name
            self.file_size = uploaded_file.size
            if uploaded_file.size > self.max_file_size:
                raise MaximumFileSizeExceeding("File too big")
            
            #logger.info("media_path : {0} - {1} - {2}".format(MEDIA_ROOT, self.upload_path, uploading_filename))
            temporary_dir = os.path.join(MEDIA_ROOT, self.upload_path)
            try:
                os.mkdir(temporary_dir)
            except OSError,ex:
                pass
            temporary_path = os.path.join(temporary_dir, uploading_filename)
            #logger.info("temporary_path : {0} ".format(temporary_path))
            dest_path = self.get_no_collition_filename(temporary_path)
            #UploadedFile
            #if self.request.session.has_key('last_file_name'):
                #is_deleted = self.delete_last_file(request)
            entirefilepath = self.save_file(request, dest_path, uploaded_file)
            filepath = os.path.relpath(entirefilepath, start=MEDIA_ROOT) 
            #logger.info("temporary_path : {0} ".format(filepath))
            request.session['last_file_name']= os.path.split(filepath)[1] 
            return (uploaded_file,filepath) or (None,None)
        raise FileNotSend("File Not Send")
    
    def post(self, request, *args, **kwargs):
        file, filepath = (None, "")
        try:
            (file, filepath) = self.process_data(request)
            msg = _("File Uploaded")
        except MaximumFileSizeExceeding as ex:
            logger.exception("MaximumFileSizeExceeding")
            msg = _(u'Dimensione del file eccessiva')
        except FileNotSend:
            logger.exception("FileNotSend")
            msg = _(u'File Non inviato')
        return self.render_to_response(self.get_context_data(object=self.object, **({'file':file,'filepath':filepath,'msg':msg})))


    def convert_context_to_json(self, context, **kwargs):
        kwargs = kwargs or {}
        kwargs.update(context)
        if context.get('file',None):
            context = self.data_valid(**kwargs)
        else:
            context = self.data_invalid(**kwargs)
        json_context = super(UploadAttachmentView,self).convert_context_to_json(context)
        return json_context

    def get_image_opts(self):
        return { 
                'crop': 'center',
                'quality' : self.image_quality
               }
        
    def get_image_geometry(self):
        return self.image_dims
    
    def get_meta_data(self):
        return {}
    
    def data_valid(self, **kwargs):
        wrapped_file = kwargs.get('file',None)
        image = kwargs.get('file',None)
        image_path = os.path.join(MEDIA_ROOT, self.upload_path, image.name)
        filename = kwargs.get('filename',None)
        msg = kwargs.get('msg',None)
        thumbnail_image = get_thumbnail(image_path, self.get_image_geometry(), **(self.get_image_opts()))
        #return { 'message': { 'text': _(u'Salvataggio eseguito con successo'), 'status': 'SUCCESS' }, 'file':file, 'jsonrpc':'2.0' }
        return {"files": [{
                           "name": wrapped_file.name,
                           "size": wrapped_file.size,
                           "url": '/',
                           "thumbnailUrl": thumbnail_image.url,
                           "deleteUrl":"/",
                           "deleteType" : "DELETER",
                           }
                        ],
                "meta" : self.get_meta_data()
                }
    
    def data_invalid(self, **kwargs):
        wrapped_file = kwargs.get('file',None)
        filename = kwargs.get('filename',None)
        msg = kwargs.get('msg',None)
        return { 'message': { 'text': msg, 'status': 'ERROR' }, 'file_size':wrapped_file.size, 'jsonrpc':'2.0' }

    def get_json_response(self, content, **httpresponse_kwargs):
        "Construct an `HttpResponse` object."
        #invece di application/json text/plain x colpa di explorer8|
        return HttpResponse(content,
                            content_type='text/plain',
                    **httpresponse_kwargs)

class UploadAttachmentModelView(UploadAttachmentView, SingleObjectMixin):
    model = None

    
    """def get_player(self):
        #player,created = Player.objects.get_or_create(id=1) 
        player = None
        raise NotImplementedError()
        return player
    
    def create_object(self, **kwargs):
        player = self.get_player()
        return self.model.objects.create(**({
                        'player' : player,
                        'original_image':kwargs.get('file',None)
                    }))"""

    def get_object_creation_kwargs(self, **kwargs):
        return kwargs
    
    def create_object(self, **kwargs):
        return self.model.objects.create(**kwargs)

    def post(self, request, *args, **kwargs):
        self.object = None
        file, filepath = (None, "")
        try:
            (file, filepath) = self.process_data(request)
            object_kwargs = self.get_object_creation_kwargs(**({
                             'file': file,
                             'filepath': filepath,
                         }))
            try:
                self.object = self.create_object(**object_kwargs)
                msg = _("Object Created")
            except Exception as ex:
                raise(ex) 
        except MaximumFileSizeExceeding as ex:
            logger.exception("MaximumFileSizeExceeding")
            msg = _(u'Dimensione del file eccessiva')
        except FileNotSend:
            logger.exception("FileNotSend")
            msg = _(u'File Non inviato')
        return self.render_to_response(self.get_context_data(object=self.object, 
                                                             **({'file':file,'filepath':filepath,'msg':msg})))
        
class UploadBase64AttachmentView(object):
    """
    Da delle utility per iol salvataggio di file:
    x aver il file chiamare e funzioni in questa maniera:
    image = self.convert_base64_to_image(self.get_base64_image())
    """
    image_kwargs = "image"
    
    def convert_base64_to_image(self, base64image):
        #logger.info(base64image)
        image = base64image.encode('ascii').replace("\\n", "").replace("\n", "")
        #logger.info(image)
        converted_image = ContentFile(base64.b64decode(str(image)))#urlsafe_b64decode(str(image)))
        return converted_image

    def get_base64_image(self):
        _post_data = self.request.POST.copy()
        try:
            base64_image = _post_data.get(self.image_kwargs)
            base64_image = base64_image.split(',')[1]
            return base64_image
        except KeyError as ex:
            logger.exception("KeyError")
            raise FileNotSend()
    
    
class UploadBase64AttachmentModelView(JSONResponseMixin, SingleObjectMixin, UploadBase64AttachmentView, View):
    image_field = 'edited_image'
    model = None
    

    def get_image_filename(self, **kwargs):
        if self.object and self.object.id:
            return "image_{0}.jpg".format(self.object.id)
        else:
            return "image_{0}.jpg".format(generate_code())

    def get_object(self, queryset=None):
        if 'pk' in self.kwargs:  # UpdateView
            obj = super(UploadBase64AttachmentModelView, self).get_object(queryset)
            return obj
        else:  # CreateView
            return None     

    def get_object_creation_kwargs(self, **kwargs):
        edited_image = self.convert_base64_to_image(self.get_base64_image())
        return {self.image_field:edited_image}
    
    def create_object(self, **kwargs):
        return self.model.objects.create(**kwargs)

    def update_object(self):
        try:
            edited_image = self.convert_base64_to_image(self.get_base64_image())
            getattr(self.object,self.image_field).save("{0}".format(self.get_image_filename()), edited_image)#), save=False)
            return True
        except FileNotSend as ex:
            logger.exception()
            return False

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        if self.object:
            self.update_object()
        else:
            self.object = self.create_object(self.get_object_creation_kwargs(**kwargs))
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)
    
    def get_meta_data(self):
        return {'imageUrl' : self.object and getattr(self.object, self.image_field) and getattr(self.object, self.image_field).url or ""}
    
    def data_valid(self, **kwargs):
        msg = kwargs.get("msg", "File salvato con successo")
        return { 'status':201, 'message': { 'text': msg, 'status': 'ok' }, 'meta':self.get_meta_data()}

    def data_invalid(self, **kwargs):
        msg = kwargs.get("msg", "Problemi nel salvataggio")
        return { 'status':304, 'message': { 'text': msg, 'status': 'ko' }}
    
    
    def convert_context_to_json(self, context, **kwargs):
        kwargs = kwargs or {}
        kwargs.update(context)
        if self.object and getattr(self.object, self.image_field):
            context = self.data_valid(**kwargs)
        else:
            context = self.data_invalid(**kwargs)
        json_context = super(UploadBase64AttachmentModelView,self).convert_context_to_json(context)
        return json_context
    
    def get_json_response(self, content, **httpresponse_kwargs):
        "Construct an `HttpResponse` object."
        #invece di application/json text/plain x colpa di explorer8|
        return HttpResponse(content,
                            content_type='text/plain',
                    **httpresponse_kwargs)