# -*- coding: utf-8 -*-
""" BestWestern  Project >  fhcore.views.thumbnail_mixins by Frankhood
    @author: Frankhood Business Solutions
    @author: Gaetano D'Onghia
"""
from __future__ import absolute_import, print_function, unicode_literals
import logging

from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext, ugettext_lazy as _

logger = logging.getLogger(__name__)



###########################################################################
#######################     THUMBNAIL MIXINS  #############################
###########################################################################

try:
    from sorl.thumbnail.shortcuts import get_thumbnail

    class SorlThumbnailPreviewMixin(object):
        image_geometry = "50x50"
        image_quality = 90
        crop = 'center'

        def get_image_geometry(self):
            return self.image_geometry

        def get_image_opts(self):
            return {
                    'crop': self.crop,
                    'quality': self.image_quality
                   }

        def get_thumbnail(self, image_path):
            return get_thumbnail(image_path, self.get_image_geometry(), **(self.get_image_opts()))

except ImportError as ex:
    logger.exception("Sorl Thumbnail is not installed")

try:
    from easy_thumbnails.files import get_thumbnailer

    class EasyThumbnailPreviewMixin(object):
        image_geometry = (50, 50)
        # image_quality = 90
        crop = 'center'
        thumbnail_storage = None

        def get_image_geometry(self):
            return self.image_geometry

        def get_image_opts(self):
            return {'size':self.get_image_geometry(),
                     'crop':'smart',
                     # 'quality' : self.image_quality
                     }

        def get_thumbnail_storage(self):
            return self.thumbnail_storage

        def get_thumbnail(self, image_path):
            thumbnailer = get_thumbnailer(image_path)
            if self.get_thumbnail_storage():
                thumbnailer.source_storage = self.get_storage()
                thumbnailer.thumbnail_storage = self.get_thumbnail_storage()
            return thumbnailer.get_thumbnail(self.get_image_opts()).url  # name

except ImportError as ex:
    logger.exception("Easy Thumbnail is not installed")