# -*- coding: utf-8 -*-
"""  fHCoreLibs.widgets.easy_thumbnail_image_widget.py   
    @author: Frankhood Business Solutions
    @date : 10/apr/2013
"""
import time
import datetime
import logging
import re
from os import path

from django.contrib.admin.widgets import AdminFileWidget
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from easy_thumbnails.exceptions import InvalidImageFormatError

logger = logging.getLogger(__name__)

try:
    from easy_thumbnails.files import get_thumbnailer
    
    class ConfigurableImageWidget(AdminFileWidget):
        """
        use it whit this statement if your admin inherits from ConfigurableWidgetsMixinAdmin
            dbfield_overrides = {'img':{'widget':ConfigurableImageWidget({'width':80,'height':80})}}
        """
        default_width = 60
        default_height = 60
        default_crop = "smart"
        
        class Media:
            css = {
                'all': ('common/css/admin_form_overrides.css',)
            }
        
        def __init__(self, attrs=None):
            if attrs is not None:
                for k in ['width','height','crop']:
                    try:
                        setattr(self,k,attrs.pop(k))
                    except KeyError:
                        setattr(self,k,None)
            super(ConfigurableImageWidget,self).__init__
            if attrs is not None:
                self.attrs = attrs.copy()
            else:
                self.attrs = {}
        
        def get_width(self):
            return getattr(self,'width',self.default_width)
        def get_height(self):
            return getattr(self,'height',self.default_height)
        def get_crop(self):
            return getattr(self,'crop',self.default_crop)

        def get_image_opts(self):
            return {'size':(self.get_width(),self.get_height()), 
                    'crop':self.get_crop()}        
        
        def get_image_html(self, value, thumbnail):
            output_html = '''
                <div class="box img-preview" style="float:left;margin-right:5px;">
                    <a target="_blank" href="%s"><img src="%s" alt="%s" /></a>
                </div>%s ''' % (value.url, thumbnail.url, path.split(value.url)[-1], _('Change:'))
            return output_html

        def render(self, name, value, attrs=None):
            output = []
            if value and hasattr(value, "url"):
                try:
                    if hasattr(self,'thumbnail') and self.thumbnail:
                        thumbnail=self.thumbnail
                    else:
                        realname = re.sub(r'^(\w+)-(\d+)-', '', name)
                        image_preview = getattr(value.instance, realname)
                        opts = self.get_image_opts()
                        thumbnail = get_thumbnailer(image_preview).get_thumbnail(opts)
                    output.append(self.get_image_html(value, thumbnail))
                except IOError:
                    pass
                except InvalidImageFormatError as ex:
                    logger.info("File con formato non previsto")
                except Exception as ex:
                    raise
            output.append(super(AdminFileWidget, self).render(name, value, attrs))
            return mark_safe(u''.join(output))
                        
    class PreviewImageWidget(AdminFileWidget):
        
        class Media:
            css = {
                'all': ('common/css/admin_form_overrides.css',)
            }
        
        def get_image_html(self, value, thumbnail):
            output_html = '''
                <div class="box img-preview" style="border:2px solid #9AB8D6;float:left;margin-right:5px;">
                    <h4 style="text-align:center;color:#fff;background:#9AB8D6;">%s</h4>
                    <a target="_blank" href="%s"><img src="%s" alt="%s" /></a>
                </div>%s ''' % (_('Preview'), value.url, thumbnail.url, path.split(value.url)[-1], _('Change:'))
            return output_html
        
        def get_image_opts(self):
            return {'size':(60,60), 'crop':'smart'}
    
        def render(self, name, value, attrs=None):
            output = []
            if value and hasattr(value, "url"):
                try:
                    realname = re.sub(r'^(\w+)-(\d+)-', '', name)
                    image_preview = getattr(value.instance, realname)
                    opts = self.get_image_opts()
                    thumbnail = get_thumbnailer(image_preview).get_thumbnail(opts)
                    output.append(self.get_image_html(value, thumbnail))
                except IOError:
                    pass
                except InvalidImageFormatError as ex:
                    logger.info("File con formato non previsto")
                except Exception as ex:
                    raise
            output.append(super(AdminFileWidget, self).render(name, value, attrs))
            return mark_safe(u''.join(output))
    
    class PreviewBigImageWidget(PreviewImageWidget):
        
        def get_image_html(self, value, thumbnail):
            output_html = '''
                <div class="box img-preview" style="float:left;margin-right:5px;">
                    <a target="_blank" href="%s"><img src="%s" alt="%s" /></a>
                </div>%s ''' % (value.url, thumbnail.url, path.split(value.url)[-1], _('Change:'))
            return output_html
        
        def get_image_opts(self):
            return {'size': (300,220), 'crop':'smart'}
    
    
    class PreviewGenericImageWidget(PreviewImageWidget):
        #DA TESTARE.. puzza di errore
        
        def render(self, name, value, attrs=None):
            output = []
            if value and hasattr(value, "url"):
                try:
                    realname = re.sub(r'^(\w+)-(\d+)-', '', name)
                    realname_splitted = realname.split('content_type-object_id-')
                    img_name=''
                    for item in realname_splitted:
                        img_name+=item
                    image_preview = getattr(value.instance, img_name)
                    opts = self.get_image_opts()
                    thumbnail = get_thumbnailer(image_preview).get_thumbnail(opts)
                    output.append(self.get_image_html(value, thumbnail))
                except IOError:
                    pass
                except InvalidImageFormatError as ex:
                    logger.info("File con formato non previsto")
                except Exception, ex:
                    raise
            output.append(super(AdminFileWidget, self).render(name, value, attrs))
            return mark_safe(u''.join(output))

except ImportError, ex:
    pass
    